#include "hauptcontroller.h"

/*
 *   Initialisiert den Controller und die angeschlossenen
 *   Komponenten.
 */
void init(void) {

    // Aktiviert USART und TWI.
    usart_init(USART_BAUD, USART_DOUBLE);
    twi_init();

    /*
        Setzt die initialen Datenrichtungen der Pins am
        Controller. Deaktiviert den Output am LCD-/EPROM-Bus
        (Port A) und am EPROM/EEPROM-Schalter (Pin D3).
        Aktiviert den Output fuer die EPROM-Adressleitungen
        (Port B und Pins D4-D6), fuer die LCD- und
        EPROM-Steuerleitungen (Pins C2-C7 und D7) und fuer
        die LED (Pin D2).
    */
    DDRA = 0x00;
    DDRB = 0xff;
    DDRC |= 0xfc;
    DDRD &= ~(1<<DDD3);
    DDRD |= 0xf4;

    /*
        Setzt die initialen Daten bzw. Pull-Up-Widerstaende.
        Deaktiviert die Pull-Up-Widerstaende am LCD-/EPROM-Bus
        (Port A) und am EPROM/EEPROM-Schalter (Pin D3). Setzt
        die EPROM-Adressleitungen (Port B und Pins D4-D6), die
        LCD-Steuerleitungen (Pins C2-C5) und die Brennleitung
        des EPROMs (Pin C7) auf 0. Setzt die Output-Leitung des
        EPROMs (Pin C6), die Brennspannungsleitung (Pin D7) und
        die LED-Leitung (Pin D2) auf 1.
    */
    PORTA = 0x00;
    PORTB = 0x00;
    PORTC &= 0x03;
    PORTC |= 0x40;
    PORTD &= 0x03;
    PORTD |= 0x84;

    // Initialisiert das LCD und den Editor.
    lcd_init();
    editor_init();
}

/*
 * Wird beim Starten des Controllers aufgerufen. Initialisiert
 * den Controller und uebergibt die Kontrolle an das
 * Hauptmenue.
 */
int main(void) {
    init();
    menu_main();
    return 0;
}
