#include "menu.h"

/*
 * Zeigt das uebergebene Menue an, indem das LCD erst geleert
 * und danach neu beschrieben wird.
 */
void menu_display(uint8_t menu) {

    // Leert das LCD, so dass spaeter nur noetige Zeichen
    // geschrieben werden muessen.
    lcd_clear();

    switch (menu) {

        case MENU_WAIT: // Warteanzeige
            lcd_write_string(LCD_LINE1, "Bitte warten");
            lcd_write_string(LCD_LINE4 + 20, "Abbruch");
            break;

        case MENU_MAIN: // Hauptmenue
            lcd_write_string(LCD_LINE1, "Hauptmenue");
            lcd_write_string(LCD_LINE4,
                    "Editor Auslesen Info   PC");
            break;

        case MENU_INFO_E_EMPTY: // EPROM ist leer
            lcd_write_string(LCD_LINE1, "Info");
            lcd_write_string(LCD_LINE2, "Das EPROM ist leer");
            lcd_write_string(LCD_LINE4, "reload");
            lcd_write_string(LCD_LINE4 + 20, "zurueck");
            break;

        case MENU_INFO_E_N_EMPTY: // EPROM ist nicht leer
            lcd_write_string(LCD_LINE1, "Info");
            lcd_write_string(LCD_LINE2,
                    "Das EPROM ist nicht leer");
            lcd_write_string(LCD_LINE4, "reload");
            lcd_write_string(LCD_LINE4 + 20, "zurueck");
            break;

        case MENU_INFO_EE_EMPTY: // EEPROM ist leer
            lcd_write_string(LCD_LINE1, "Info");
            lcd_write_string(LCD_LINE2, "Das EEPROM ist leer");
            lcd_write_string(LCD_LINE4, "reload");
            lcd_write_string(LCD_LINE4 + 20, "zurueck");
            break;

        case MENU_INFO_EE_N_EMPTY: // EEPROM ist nicht leer
            lcd_write_string(LCD_LINE1, "Info");
            lcd_write_string(LCD_LINE2,
                    "Das EEPROM ist nicht leer");
            lcd_write_string(LCD_LINE4, "reload loeschen");
            lcd_write_string(LCD_LINE4 + 20, "zurueck");
            break;

        case MENU_INFO_CLEARING_EE: // EEPROM wird geloescht
            lcd_write_string(LCD_LINE1, "Info");
            lcd_write_string(LCD_LINE2,
                    "Das EEPROM wird geloescht");
            break;

        case MENU_PC: // PC-Menue
            lcd_write_string(LCD_LINE1, "PC");
            lcd_write_string(LCD_LINE4 + 20, "zurueck");
            break;

        // Menuezeile des Editors (den Rest schreibt der Editor
        // selbst):
        case MENU_EDITOR:
            lcd_write_string(LCD_LINE4, "Leeren Brennen");
            lcd_write_string(LCD_LINE4 + 20, "zurueck");
            break;

        case MENU_BURN: // Brenn-Bestaetigung
            lcd_write_string(LCD_LINE1, "Brennen");
            lcd_write_string(LCD_LINE2,
                    "Zum Brennen OK druecken");
            lcd_write_string(LCD_LINE4 + 20, "Abbruch");
            break;

        case MENU_BURNING: // EPROM wird gebrannt
            lcd_write_string(LCD_LINE1, "Brennen");
            lcd_write_string(LCD_LINE2,
                    "Das EPROM wird gebrannt");
            break;

        case MENU_BURNED: // EPROM wurde gebrannt
            lcd_write_string(LCD_LINE1, "Brennen");
            lcd_write_string(LCD_LINE2,
                    "Das EPROM wurde gebrannt");
            lcd_write_string(LCD_LINE4 + 20, "zurueck");
            break;

        default: // Kein gueltiges Menue
            misc_error();
            break;

    }

}

/*
 * Wartet auf einen Tastendruck und gibt diesen zurueck.
 * Zwischen den Tasten-Abfragen wird das PC-Protokoll
 * abgearbeitet.
 */
uint8_t menu_get_keystroke(void) {
    uint8_t key = KEY_NONE;
    uint8_t i = 0;

    // Erst beenden, wenn eine Taste gedrueckt wurde.
    while (key == KEY_NONE) {

        // Nur alle 10ms Tastendruecke abfragen, in der
        // Zwischenzeit das PC-Protokoll abarbeiten.
        for (i = 0; i < 10; i++) {
            pc_process();
            _delay_ms(1);
        }

        // Falls ein neuer Tastendruck anliegt, die Taste lesen
        // (Ist nur im Fehlerfall false, da bei keiner
        // gedrueckten Taste die Taste KEY_NONE uebertragen
        // wird).
        if (twi_read(2) == TWI_GET_DATA) {
            key = twi_get_data();
        }

    }

    return key;
}

/*
 * Zeigt das Info-Menue an und ermoegicht ein EEPORM zu leeren.
 */ 
uint8_t menu_info(void) {
    uint8_t menu_ret = 0;

    // Speichert, ob ein nicht-leeres EEPROM eingesteckt ist.
    uint8_t ee_n_empty = 0;

    while (menu_ret == 0 || menu_ret == 0xff) {

        if (menu_ret < 0xff) {

            // Warte-Anzeige
            menu_display(MENU_WAIT);

            if (eprom_is_empty()) { // Leeres (E)EPROM

                if (eprom_get_type() == EPROM_NEPROM) {
                        // Leeres EPROM
                    menu_display(MENU_INFO_E_EMPTY);
                } else { // Leeres EEPROM
                    menu_display(MENU_INFO_EE_EMPTY);
                }

            } else { // Nicht-leeres (E)EPROM

                if (eprom_get_type() == EPROM_NEPROM) {
                        // Nicht-leeres EPROM
                    menu_display(MENU_INFO_E_N_EMPTY);
                } else { // Nicht-leeres EEPROM
                    menu_display(MENU_INFO_EE_N_EMPTY);
                    ee_n_empty = 1;
                }

            }

            menu_ret = 0xff;
        }

        switch (menu_get_keystroke()) {

            case KEY_SK1: // Aktualisieren
                menu_ret = 0;
                break;

            case KEY_SK2: // EEPROM loeschen

                // Nur nicht-leere EEPROMs loeschen.
                if (ee_n_empty) {
                    menu_display(MENU_INFO_CLEARING_EE);
                    eprom_clear_eeprom();
                }

                menu_ret = 0;
                break;

            case KEY_SK4: // Zurueck
                menu_ret = 1;
                break;

        }

    }

    return menu_ret - 1;
}

/*
 * Zeigt das PC-Menue und startet bzw. stoppt das PC-Protokoll.
 */
uint8_t menu_pc(void) {
    uint8_t menu_ret = 0;

    while (menu_ret == 0 || menu_ret == 0xff) {

        if (menu_ret < 0xff) {
            menu_display(MENU_PC);

            // Starte das PC-Protokoll.
            pc_start();

            menu_ret = 0xff;
        }

        switch (menu_get_keystroke()) {

            case KEY_SK4: // Zurueck

                // Stoppt das PC-Protokoll.
                pc_stop();

                menu_ret = 1;
                break;

        }

    }

    return menu_ret - 1;
}

/*
 * Zeigt das Brennmenue und brennt das EPROM.
 */
uint8_t menu_burning(void) {
    uint8_t menu_ret = 0;

    while (menu_ret == 0 || menu_ret == 0xff) {

        if (menu_ret < 0xff) {

            // Zeigt die Brennmeldung an, brennt das EPROM und
            // zeigt abschliessend die Gebrannt-Meldung an.
            menu_display(MENU_BURNING);
            editor_burn();
            menu_display(MENU_BURNED);

            menu_ret = 0xff;
        }

        switch (menu_get_keystroke()) {

            case KEY_SK4: // Zurueck zum Editor
                menu_ret = 2;
                break;

        }

    }

    return menu_ret - 1;
}

/*
 * Zeigt das Brennbestaetigungsmenue.
 */
uint8_t menu_burn(void) {
    uint8_t menu_ret = 0;

    while (menu_ret == 0 || menu_ret == 0xff) {

        if (menu_ret < 0xff) {
            menu_display(MENU_BURN);
            menu_ret = 0xff;
        }

        switch (menu_get_keystroke()) {

            case KEY_OK: // Brennen
                menu_ret = menu_burning();
                break;

            case KEY_SK4: // Zurueck
                menu_ret = 1;
                break;

        }

    }

    return menu_ret - 1;
}

/*
 * Zeigt den Editor an.
 */
uint8_t menu_editor(void) {
    uint8_t menu_ret = 0;
    uint8_t key = KEY_NONE;

    while (menu_ret == 0 || menu_ret == 0xff) {

        if (menu_ret < 0xff) {
            menu_display(MENU_EDITOR);

            // Zeigt Cursor und Editor an.
            lcd_display_cursor(1);
            editor_display();

            menu_ret = 0xff;
        }

        key = menu_get_keystroke();

        switch (key) {

            case KEY_SK1: // Editor leeren
                editor_clear();
                menu_ret = 0;
                break;

            case KEY_SK2: // Brennen

                // Cursor abschalten.
                lcd_display_cursor(0);
                menu_ret = menu_burn();
                break;

            case KEY_SK3: // Unbenutzt
                break;

            case KEY_SK4: // Zurueck
                menu_ret = 1;
                break;

            default: // Editortaste wurde gedrueckt
                editor_handle_keystroke(key);
                break;

        }

    }

    // Cursor abschalten.
    lcd_display_cursor(0);
    return menu_ret - 1;
}

/*
 * Zeigt das Hauptmenue in einer Endlosschleife an und wartet
 * auf einen Tastendruck. Wird ein Softkey gedrueckt, wird das
 * entsprechende Untermenue aufgerufen.
 */
uint8_t menu_main(void) {
    uint8_t menu_ret = 0;

    while (1) {

        if (menu_ret < 0xff) {
            menu_display(MENU_MAIN);
            menu_ret = 0xff;
        }

        switch (menu_get_keystroke()) {

            case KEY_SK1: // Editor
                menu_ret = menu_editor();
                break;

            case KEY_SK2: // Auslesen
                menu_display(MENU_WAIT);
                editor_read_eprom();
                menu_ret = menu_editor();
                break;

            case KEY_SK3: // Info
                menu_ret = menu_info();
                break;

            case KEY_SK4: // PC
                menu_ret = menu_pc();
                break;

        }

    }

    return 0;
}
