#include "usart.h"

/*
 * Sendet ein Zeichen (1 Byte) ueber RS232
 */
int usart_write(char data, FILE *stream) {

    // Warte, bis das letzte Byte komplett versendet wurde.
    while (!(UCSR0A & (1<<UDRE0)));

    // Setze das zu sendende Zeichen.
    UDR0 = data;
    return 0;
}

// Datei-Deskriptor, der als stdout benutzt wird und ueber
// RS232 sendet.
FILE usart_out = FDEV_SETUP_STREAM(usart_write, NULL,
        _FDEV_SETUP_WRITE);

/*
 * Initialisiert RSR232 und setzt stdout zum Senden ueber 
 * RS232.
 */
void usart_init(uint16_t baud, uint8_t u2x) {

    // Divisor bei der Berechnung des Wertes des Baud Rate
    // Registers.
    uint16_t divisor = 16;

    // Wenn die doppelte Baud Rate benutzt wird, muss der
    // Divisor halbiert werden.
    if (u2x == 1) {
        divisor = 8;
        UCSR0A |= (1<<U2X0);
    }

    // Berechnen des Wertes des Baud Rate Registers.
    uint16_t ubrr = F_CPU / divisor / baud - 1;

    // Baud Rate Register setzen.
    UBRR0H = (ubrr & 0x0f00)>>8;
    UBRR0L = ubrr & 0x00ff;

    // USART aktivieren.
    UCSR0B = (1<<RXEN0) | (1<<TXEN0);

    // Setze neuen stdout.
    stdout = &usart_out;
}

/*
 * Gibt zurueck, ob ein Zeichen empfangen wurde.
 */
uint8_t usart_has_data(void) {

    if ((UCSR0A & (1<<RXC0))) { // Zeichen empfangen
        return 1;
    } else { // Kein Zeichen empfangen
        return 0;
    }

}

/*
 * Wartet, bis ein Zeichen empfangen wurde und gibt dies
 * zurueck. Achtung, wenn kein Zeichen empfangen wird, wird
 * diese Funktion nie beendet.
 */
uint8_t usart_read_char(void) {

    while (!usart_has_data());

    return UDR0;
}

/*
 * Sendet ein einzelnes Zeichen.
 */
void usart_write_char(uint8_t data) {
    usart_write(data, NULL);
}

/*
 * Sendet einen String, indem es jedes Zeichen einzeln sendet.
 */
void usart_write_string(uint8_t str[]) {
    uint8_t i = 0;

    // 0x00 beendet einen String, also dann abbrechen.
    while (str[i] != 0) {
        usart_write_char(str[i]);
        i++;
    }

}
